<?php
/*
* Define shortcodes for event calendar
* Uses the jQuery Fullcalendar
*
*/

namespace Tekomatik\HelloEvent;
  
class Hello_Event_Calendar {
  
  public $cf_calendar_add_my_resources = false;
  
  public function __construct() {
    add_action('wp_enqueue_scripts', array($this, 'calendar_register_my_resources'));
    add_action('wp_footer', array($this, 'calendar_insert_my_resources_if_necessary'));
    add_action('wp_ajax_hello_event_get_events', array($this, 'get_events'));
    add_action('wp_ajax_nopriv_hello_event_get_events', array($this, 'get_events'));
    add_shortcode( 'hello-event-calendar', array($this, 'handle_shortcode_calendar'));
  }


// ================== Function respoding to Ajax returning the events for a period ============================
// To make this work:
// 1) Define the function
// 2) add action to connect wp_ajax with the function
// 3) make a call to wp_localize_script to define adminAjax as the URL to be called
// 4) Set up the ajax call in js with url = adminAjax and action = name of the function to call


function get_events() {
  // Called via ajax
  global $hello_event_object; // Will give us access to its methods
  // debug_log("AJAX GET_EVENTS HERE");
  // debug_log(print_r($_REQUEST, true));
  remove_filter( 'the_title', 'wptexturize' ); // Stop WP from converting quotes into html quotes
  $start = $_REQUEST['start'] ;
  $end = $_REQUEST['end'] ;
  // Fullcalendar needs 1 day to be added to the end date
  $fix_end_date = isset($_REQUEST['fix_end_date']) ? $_REQUEST['fix_end_date'] : false;
  $cf_query = array(
    'post_type' => HELLO_EVENT::EVENT_SLUG,
    'post_status' => 'publish',
    'meta_query' => array(
      array(
        'key' => 'start_date',
        'value' => $start,
        'compare' => '>='
      ),
      array(
        'key' => 'start_date',
        'value' => $end,
        'compare' => '<='
      )
    ),
    'nopaging' => true
  );
  // debug_log("Query 0 = " . print_r($cf_query,true));
  $cf_query = apply_filters('hello_event_calendar_get_events', $cf_query); // Allow add-ons to intercept
  // debug_log("Query 1 = " . print_r($cf_query,true));
  global $post;
  $the_posts = new \WP_Query($cf_query);
  // debug_log("Result: " . print_r($the_posts, true));
  $events = array();
  while($the_posts->have_posts()) {
    $the_posts->the_post();
    $ev = array();
    $id = get_the_ID();
    // debug_log("On with ID = " . $id);
    $url = get_permalink($id);
    $start_date = get_post_meta( $id, 'start_date', true );
    $end_date = get_post_meta( $id, 'end_date', true );
    $start_time = get_post_meta( $id, 'start_time', true );
    $end_time = get_post_meta( $id, 'end_time', true );
    if ( ($start_time == "00:00") && ($end_time == "00:00") )
      $start_time = "Full day";
    else
      $start_time = $hello_event_object->transform_iso_time($start_time);
    $location = get_post_meta( $id, 'location', true );
    $dates_html = "<div class='date'>$start_date - $end_date</div>";
    $start_html = "<div class='time'>$start_time</div>";
    $location_html = "<div class='location'>$location</div>";
    $see_more = '<div class="clearfix"><a href="'.$url.'" class="btn btn-success pull-right">'.__("See more", "hello_event").'</a>'; 
    $ev['title_long'] = get_the_title()." - ".$hello_event_object->transform_iso_date(get_post_meta( $id, 'start_date', true ));
    $ev['title'] = get_the_title();
    $ev['start'] = get_post_meta( $id, 'start_date', true );
    if ( get_post_meta( $id, 'heure', true) ) { $ev['start'] .= "T".get_post_meta( $id, 'heure', true); };
    $ev['end'] = get_post_meta( $id, 'end_date', true );
    if ( !$ev['end'] ) { $ev['end'] = get_post_meta( $id, 'start_date', true ); }
    // For the end date to display correctly in fullcalendar we need to add 1 day to the end date!
    if ($fix_end_date) {
      $end_fixed_date = new \DateTime($ev['end']);
      $end_fixed_date->modify('+1 day');
      $ev['end'] = $end_fixed_date->format('Y-m-d');
    }
    
    //$excerpt = apply_filters('the_excerpt', get_post_field('post_excerpt', $post->ID));
    //$ev['description'] = $excerpt . $see_more ;
    $ev['description'] = "<div class='event-modal-description'>" . $dates_html . $start_html . $location_html . $hello_event_object->real_or_computed_excerpt($id) . $see_more ."</div>";
    $ev['url'] = $url;
    $ev = apply_filters('event-information', $ev, $id); // Allow other plugin to complement the event information
    array_push($events, $ev);
  };
  wp_reset_postdata();
  $events_to_test = array(array(
    "title_long" =>"Stage cuisine 2019 - 2019-10-01",
    "title"=>"Stage cuisine 2019",
    "start"=>"2019-10-01",
    "end"=>"2019-10-02",
    "description"=>"Villard de Lans, France."
  ));
  $events = json_encode($events);
  echo $events;
  wp_die();
}



  function calendar_register_my_resources() {
    wp_register_script( 'moment',
        plugins_url('fullcalendar/lib/moment.min.js', __FILE__), ['jquery'], '', true );
    wp_register_script( 'fullcalendar',
        plugins_url('fullcalendar/fullcalendar.js', __FILE__), ['jquery', 'moment'], '', true );
    wp_register_script( 'hello_event_fullcalendar_french',
        plugins_url('fullcalendar/lang/fr.js', __FILE__), ['jquery', 'moment'], '', true );
    wp_register_script( 'hello_event_fullcalendar_swedish',
        plugins_url('fullcalendar/lang/sv.js', __FILE__), ['jquery', 'moment'], '', true );
    wp_register_script( 'hello_event_bootbox',
        plugins_url('js/bootbox.min.js', __FILE__), ['jquery'], '', true );
    wp_register_style( 'jquery_dialog', plugins_url('js/jquery-ui-1.12.1-2.custom-dialog/jquery-ui.min.css', __FILE__));
    wp_register_style( 'hello_event_frontend-calendar', plugins_url('css/frontend-calendar.css', __FILE__));
    wp_register_style( 'fullcalendar', plugins_url('fullcalendar/fullcalendar.css', __FILE__));
    // Prepare for Ajax
    wp_localize_script('fullcalendar', 'adminAjax', [admin_url('admin-ajax.php')]);
    $this->cf_calendar_add_my_resources = false;
  }

  function calendar_insert_my_resources_if_necessary() {
    // The resources are only included if the shortcode is used on the page   
    // debug_log("Do we need to insert resources for the calendar?"); 
    if ( ! $this->cf_calendar_add_my_resources )
      return;
    // debug_log("Yes we do");
    $modals = isset(get_option( 'hello_event' )['hello_field_modal']) ? get_option( 'hello_event' )['hello_field_modal'] : 'none';
    wp_enqueue_style( 'fullcalendar');
    wp_enqueue_style( 'hello_event_frontend-calendar');
    wp_enqueue_script( 'moment');
    wp_enqueue_script( 'fullcalendar');
    wp_enqueue_script( 'hello_event_fullcalendar_french');
    wp_enqueue_script( 'hello_event_fullcalendar_swedish');
    if ($modals == 'boot') {
      wp_enqueue_script( 'hello_event_bootbox');
    }
    elseif ($modals == "jquery") {
      wp_enqueue_style( 'jquery_dialog');
      wp_enqueue_script( 'jquery-ui-dialog');
    }
    
  }

  public function handle_shortcode_calendar( $args ){
    // debug_log("args=".print_r($args, true));
    $args = shortcode_atts( array(
           'header' => "{}",
       ), $args );
    
    global $hello_event_object; // Will give us access to its methods
    $modals = isset(get_option( 'hello_event' )['hello_field_modal']) ? get_option( 'hello_event' )['hello_field_modal'] : 'none';
    $locale = explode("_", get_locale())[0];
    if (!in_array($locale, Hello_Event::LOCALES)) {$locale='en';} // Fallback for esoteric locales
    $this->cf_calendar_add_my_resources = true;
    if ($modals == 'boot') {
      $evrender = "
      function (event, element) {
        element.attr('href', 'javascript:void(0);');
        element.click(function() {
          bootbox.alert({
            message: ''+event.description,
            title: event.title_long,
          });
        });
      }";
    }
    elseif ($modals == 'jquery') {
      $evrender = "
      function (event, element) {
        element.attr('href', 'javascript:void(0);');
        element.click(function() {
          
          $('#dialog').dialog('option', 'title', event.title_long);
          $('#dialog').html(''+event.description);
          $('#dialog').dialog('open');
          
        });
      }";
    }
    elseif ($modals == 'none') {
      $evrender = "
      function (event, element) {
        element.attr('href', event.url);
        }";   
    }
    $html = '';
    if ($modals == 'jquery') {
      $html .= '<div id="dialog" title="Event">Body</div>';
    }
    
    
    $html .=  '<div id="cf_calendar"></div>';
    $html .= '<script>';
    $html .= '(function($){';
    $html .= '$(document).ready(function () {';
    if ($modals == "jquery") {
       $html .= 'jQuery("#dialog").dialog({ autoOpen: false });';
       $html .= "$('#dialog').dialog('option', 'width', 330);";
       $html .= "$('#dialog').dialog('option', 'show', true);";
     }
     $header = $args['header'];
     // debug_log("header=".$header);
     // $html .= '    alert("adminajax="+adminAjax);';
    $html .= '$("#cf_calendar").fullCalendar({
      timeFormat: " ",
      firstDay:1,
      height: "auto",
      lang:"' . $locale .'",
      header:' . $header .',
      eventRender: '. $evrender.',
      events: function(start, end, timezone, callback) {
        //alert("start= " + start.format() + " end= " + end.format() + " callback= " + callback);
        jQuery.ajax({
              url: adminAjax,
              type: "POST",
              dataType: "json",
              data: {
                action: "hello_event_get_events",
                start: start.format(),
                end: end.format()
              },
              error: function(obj, status, err) {
                alert("AJAX ERROR. status:" + status + " error: " + err);
              },
              success: function(doc) {
                callback(doc);
              }
          });
      }
      })';
    $html .= '});';
    $html .= ' })(jQuery); ';
    $html .= '</script>';
    return $html;
  }
} // End of class
$hello_event_calendar_object = new Hello_Event_Calendar;
?>
